<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Facebook Connect Service
 *
 * @author Sergey Kambalin <greyexpert@gmail.com>
 * @package ow_plugins.fbconnect.bol
 * @since 1.0
 */
require_once OW_DIR_LIB . 'facebook-platform' . DS . 'facebook.php';

class FBCONNECT_BOL_Service extends FBCONNECT_BOL_ServiceBase
{
    private static $classInstance;
    
    /**
     * Returns class instance
     *
     * @return FBCONNECT_BOL_Service
     */
    public static function getInstance()
    {
        if ( null === self::$classInstance )
        {
            self::$classInstance = new self();
        }
        
        return self::$classInstance;
    }
    
    
    private $jsInitialized = false;
    
    private $urlList = array();
    
    /**
     * 
     * @var FBCONNECT_BOL_FieldDao
     */
    private $fieldDao;
    
    private $permissionList = array(
        'required' => array('email'),
        'optional' => array()
    );
    
    /**
     * Class constructor
     *
     */
    protected function __construct()
    {
        parent::__construct();
        
        $this->fieldDao = FBCONNECT_BOL_FieldDao::getInstance();
    }
    
    private function getSessionKey()
    {
        return empty($_COOKIE["{$this->getFaceBookAccessDetails()->apiKey}_session_key"]) 
            ? null 
            : $_COOKIE["{$this->getFaceBookAccessDetails()->apiKey}_session_key"];
    }
    
    private function getSessionSecret()
    {
        return empty($_COOKIE["{$this->getFaceBookAccessDetails()->apiKey}_ss"]) 
            ? null 
            : $_COOKIE["{$this->getFaceBookAccessDetails()->apiKey}_ss"];
    }
    
    public function initializeJs($requiredPermissions = array())
    {
        if ($this->jsInitialized)
        {
            return;
        }
        $document = OW::getDocument(); 
        
        $document->addScript($this->getFaceBookAccessDetails()->staticFbUrl . '/js/api_lib/v0.4/FeatureLoader.js.php');
        $document->addScript(OW::getPluginManager()->getPlugin('fbconnect')->getStaticJsUrl() . 'fb.js');
        
        // FB js object definition
        $options = new stdClass();
        if (!empty($requiredPermissions))
        {
            $options->permsToRequestOnConnect = implode(', ', $requiredPermissions);
        }
        
        $js = UTIL_JsGenerator::newInstance()
            ->callFunction(array('FB', 'init'), array($this->getFaceBookAccessDetails()->apiKey, $this->getFaceBookAccessDetails()->xdReceiverUrl, $options));
        $document->addOnloadScript((string) $js);
        
        $uri = OW::getRequest()->getRequestUri();

        $loginRouteUrl = OW::getRouter()->urlForRoute('fbconnect_login');
        $synchronizeRouteUrl = OW::getRouter()->urlForRoute('fbconnect_synchronize');
        
        // OW_FB js object definition
        $options = array(
            'onLoginUrl' => OW::getRequest()->buildUrlQueryString($loginRouteUrl, array('backUri' => urlencode($uri))),
            'onSynchronizeUrl' => OW::getRequest()->buildUrlQueryString($synchronizeRouteUrl, array('backUri' => urlencode($uri))),
        );
        
        $js->clear()->newObject(array('window', 'OW_FB'), 'OW_FBConstructor', array($options));
        $document->addOnloadScript((string) $js);
        
        $this->jsInitialized = true;
    }
    
    public function fbRequireSession($requiredPermissions = array()) 
    {
        $this->getFaceBook()->require_login($requiredPermissions);
    }
    
    public function fbGetLoggedInUser()
    {
        return $this->getFaceBook()->api_client->users_getLoggedInUser();
    }
    
    public function fbExpireSession()
    {
        $this->getFaceBook()->expire_session();
    }
    
    public function fbGetFieldValueList($fbUserId, array $fields)
    {
        $fbUserId = (int) $fbUserId;
        if (!$fbUserId)
        {
            throw new InvalidArgumentException('Invalid Argument $fbUserId');
        }
        
        if (empty($fields))
        {
            return array();
        }
        
        $fieldStr = implode(', ', $fields);
        $query = "SELECT $fieldStr FROM user WHERE uid=$fbUserId";
        
        $out = $this->getFaceBook()->api_client->fql_query($query);
        
        return $out[0];
    }
    
    
    public function fbCheckPermissionList($fbUserId, array $permissionKeys)
    {
        $permissions = $this->fbGetPermissionList($fbUserId, $permissionKeys);
        
        return !in_array($permissions, 0);
    }
    
    public function fbGetPermissionList($fbUserId, array $permissionKeys)
    {
        $fbUserId = (int) $fbUserId;
        if (!$fbUserId)
        {
            throw new InvalidArgumentException('Invalid Argument $fbUserId');
        }
        
        if (empty($permissionKeys))
        {
            return array();
        }
        
        $permissionKeyStr = implode(', ', $permissionKeys);
        $query = "SELECT $permissionKeyStr FROM permissions WHERE uid=$fbUserId";
        
        $out = $this->getFaceBook()->api_client->fql_query($query);
        
        return $out[0];
    }
    
    public function requestQuestionValueList($fbUserId, $questionNameList = null)
    {
        $fieldDtoList = empty($questionNameList) 
            ? $this->fieldDao->findAll() 
            : $this->fieldDao->findListByQuestionList($questionNameList);
        
        $converterList = array();    

        $fbFields = array();
        foreach ($fieldDtoList as $fieldDto)
        {
            /* @var $fieldDto FBCONNECT_BOL_Field */
            $fbFields[$fieldDto->fbField] = $fieldDto->fbField;
        }
        $fbFields = array_values($fbFields);
        
        $fbFieldValues = $this->fbGetFieldValueList($fbUserId, $fbFields);
        
        $out = array();
        foreach ($fieldDtoList as $fieldDto)
        {
            /* @var $fieldDto FBCONNECT_BOL_Field */
            $class = $fieldDto->converter;
            if (empty($converterList[$class]))
            {
                $converter = new $class();    
            }
            $out[$fieldDto->question] = $converter->convert($fieldDto->question, $fieldDto->fbField, $fbFieldValues[$fieldDto->fbField]);
        }
        
        return $out;
    }
    
    public function getOWQuestionDtoList()
    {
        $aliases = $this->findAliasList();
        
        $questions = BOL_QuestionService::getInstance()->findAllQuestions();
        
        $out = array();
        foreach ($questions as $question)
        {
            /* @var $question BOL_Question */
            $isText = in_array($question->presentation, array('text', 'textarea'));
            $hasAlias = !empty($aliases[$question->name]);
            
            if ($isText || $hasAlias)
            {
                $out[] = $question;
            }
        }
        
        return $out;
    }
    
    public function getPossibleFbFieldList($questionName = null)
    {
        switch ($questionName)
        {
            case 'username':
                return array('name');
            case 'email':
                return array('email');
            case 'birthdate':
                return array('birthday_date');
        }
        
        return array('first_name', 'last_name', 'name', 'interests', 'music',
                     'tv', 'movies', 'books', 'quotes', 'about_me', 'website');        
    }
    
    public function assignQuestion($question, $fbField, $converter = 'FBCONNECT_FC_TextFieldConverter')
    {
        $fieldDto = $this->fieldDao->findByQuestion($question);
        if ($fieldDto === null)
        {
            $fieldDto = new FBCONNECT_BOL_Field();            
        }
        
        $fieldDto->question = $question;
        $fieldDto->fbField = $fbField;
        $fieldDto->converter = $converter;
        
        $this->fieldDao->save($fieldDto);
    }
    
    public function findAliasDtoList()
    {
        return $this->fieldDao->findAll();
    }
    
    public function findAliasList()
    {
        $out = array();
        $aliases = $this->findAliasDtoList();
        foreach($aliases as $alias)
        {
            $out[$alias->question] = $alias->fbField;     
        }
        
        return $out;
    }
}