<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Forum edit topic action controller
 *
 * @author Egor Bulgakov <egor.bulgakov@gmail.com>
 * @package ow.ow_plugins.forum.controllers
 * @since 1.0
 */
class FORUM_CTRL_Search extends OW_ActionController
{
    private $forumService;
    
    public function __construct()
    {
        parent::__construct();
        
        $this->forumService = FORUM_BOL_ForumService::getInstance();
    }
    

    /**
     * Controller's default action
     * 
     * @param array $params
     */
    public function inForums( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('forum');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        if ( !empty($params['token']) )
        {
            $token = urldecode(htmlspecialchars($params['token']));
            $this->assign('token', $token);
            
            $page = isset($_GET['page']) ? $_GET['page'] : 1;
            $topics = $this->forumService->searchInGroups($token, $page);
            
            $authors = array();
            if ( $topics )
            {
                foreach ( $topics as &$topic )
                {
                    $topic['toolbar'] = array();
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
            }
            $this->assign('topics', $topics);
            
            // Paging
            $total = $this->forumService->countFoundTopicsInGroups($token);
            $perPage = $this->forumService->getTopicPerPageConfig();
            $pages = (int) ceil($total / $perPage);
            $paging = new BASE_CMP_Paging($page, $pages, $perPage);
            $this->assign('paging', $paging->render());
            
            $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
        }
        
        $this->addComponent('search', new FORUM_CMP_ForumSearch(
            array('scope' => 'all_forum', 'token' => $token))
        );
        
        OW::getDocument()->setHeading(OW::getLanguage()->text('forum', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_forum');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'forum', 'forum');
    }
    
    public function inGroup( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('forum');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        if ( !empty($params['token']) )
        {
            $token = urldecode(htmlspecialchars($params['token']));
            $this->assign('token', $token);
            
            $page = isset($_GET['page']) ? $_GET['page'] : 1;
            $groupId = (int)$params['groupId'];
            $isHidden = $this->forumService->groupIsHidden($groupId);
            
            if ( $isHidden )
            {
                $forumGroup = $this->forumService->findGroupById($groupId);
                $forumSection = $this->forumService->findSectionById($forumGroup->sectionId);
            
                $event = new OW_Event('forum.find_forum_caption', array('entity' => $forumSection->entity, 'entityId' => $forumGroup->entityId));
                OW::getEventManager()->trigger($event);
    
                $eventData = $event->getData();
                $componentForumCaption = $eventData['component'];
    
                $this->addComponent('componentForumCaption', $componentForumCaption);
            }
            
            $topics = $this->forumService->searchInGroups($token, $page, $groupId, $isHidden);
            
            $authors = array();
            if ( $topics )
            {
                foreach ( $topics as &$topic )
                {
                    if ( !in_array($topic['userId'], $authors) )
                    {
                        array_push($authors, $topic['userId']);
                    }
                        
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
                
                $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
            }
            
            $this->assign('topics', $topics);
            
            // Paging
            $total = $this->forumService->countFoundTopicsInGroups($token, $groupId, $isHidden);
            $perPage = $this->forumService->getTopicPerPageConfig();
            $pages = (int) ceil($total / $perPage);
            $paging = new BASE_CMP_Paging($page, $pages, $perPage);
            $this->assign('paging', $paging->render());
        }
        
        $this->addComponent('search', new FORUM_CMP_ForumSearch(
            array('scope' => 'group', 'token' => $token, 'groupId' => $groupId))
        );
        
        OW::getDocument()->setHeading(OW::getLanguage()->text('forum', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_forum');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'forum', 'forum');
    }
    
    public function inTopic( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('forum');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        if ( !empty($params['token']) )
        {
            $token = urldecode(htmlspecialchars($params['token']));
            $this->assign('token', $token);
            
            $topicId = (int)$params['topicId'];
            
            $page = isset($_GET['page']) ? $_GET['page'] : 1;
            $topics = $this->forumService->searchInTopic($token, $topicId, $page);
            
            $topic = $this->forumService->findTopicById($topicId);
            $forumGroup = $this->forumService->findGroupById($topic->groupId);
            $forumSection = $this->forumService->findSectionById($forumGroup->sectionId);
            
            if ( $forumSection && $forumSection->isHidden )
            {
                $event = new OW_Event('forum.find_forum_caption', array('entity' => $forumSection->entity, 'entityId' => $forumGroup->entityId));
                OW::getEventManager()->trigger($event);
    
                $eventData = $event->getData();
                $componentForumCaption = $eventData['component'];
    
                $this->addComponent('componentForumCaption', $componentForumCaption);
            }
            
            if ( $topics )
            {
                $authors = array();
                foreach ( $topics as &$topic )
                {
                    if ( !in_array($topic['userId'], $authors) )
                    {
                        array_push($authors, $topic['userId']);
                    }
                        
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
                
                $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
            }
            
            $this->assign('topics', $topics);
        }
        
        $this->addComponent('search', new FORUM_CMP_ForumSearch(
            array('scope' => 'topic', 'token' => $token, 'topicId' => $topicId))
        );
        
        OW::getDocument()->setHeading(OW::getLanguage()->text('forum', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_forum');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'forum', 'forum');
    }
}
